package unicrawler.crawler;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import unicrawler.CrawlerEnvironment;
import unicrawler.types.CrawlerURL;
import unicrawler.types.Page;

public class CrawlerWorker implements Runnable {

	private Logger logger = Logger.getLogger(CrawlerWorker.class.getName());
	
	private CrawlerWorkerCounter mWorkerCounter;
	private ScheduledThreadPoolExecutor mThreadPool;
	private CrawlerEnvironment mEnvironment;
	
	private CrawlerURL mUrl;
	
	public CrawlerWorker(CrawlerEnvironment _env, ScheduledThreadPoolExecutor _pool, 
			CrawlerWorkerCounter _counter, CrawlerURL _url) {
		
		this.mEnvironment = _env;
		this.mThreadPool = _pool;
		this.mWorkerCounter = _counter;
		this.mUrl = _url;
		
		this.mWorkerCounter.addCrawlerCount();
	}
	
	@Override
	public void run() {
		// Lets start doing this crawler buisness...
		try {
			
			boolean seedOnly = Boolean.parseBoolean(this.mEnvironment.getProperties().get("crawler.seedonly", "false"));
			
			// This is temporary - will be contained in specialized interface soon
			if (seedOnly && ! this.mUrl.getDomain().equals(this.mEnvironment.getSeedUrls().getDomain())) {			
				// Skip this page because it's not matching the seed url
				logger.info("Not crawling page [" + this.mUrl.getLink() + "] because it does not match seed domain [" + this.mEnvironment.getSeedUrls().getDomain());
			} else {
				logger.info("Crawling page [" + this.mUrl.getLink() + "] with depth [" + this.mUrl.getDepth() + "]");
			
				// Call the downloader
				Page crawledPage = this.mEnvironment.getDownloader().downloadPage(this.mUrl);
				
				if (crawledPage.getPageResponseCode() != 200) {
					// 200 means OK - this page isn't okay
					logger.error("URL [" + crawledPage.getPageUrl().getLink() + "] had status code [" + crawledPage.getPageResponseCode() + "]");
				}
				
				crawledPage.parsePageLinks();
				
				// Schedule page links, checking depth
				for (String link : crawledPage.getPageLinks()) {
					logger.info("Discovered link " + link);
					
					CrawlerURL child = new CrawlerURL(link, (this.mUrl.getDepth() + 1));
					if (child.getDepth() > this.mEnvironment.getProperties().getInt("crawler.maxdepth",3)) {
						logger.info("Max depth has been reached. url [" + link + "] will not be crawled");
					}
					else {
						logger.info("Depth is " + child.getDepth());
						this.mThreadPool.schedule(new CrawlerWorker(this.mEnvironment, this.mThreadPool, this.mWorkerCounter, child),
								this.mEnvironment.getProperties().getInt("thread.pooldelay", 1000),
								TimeUnit.MILLISECONDS
							);
					}
				}
				
				/* TODO: stuff like specialized frontier(s) (storage), 
				 * link normalization can go here, will be added for more advanced version
				 */
			}
		}
		finally {
			// This worker has finished executing!
			this.mWorkerCounter.minusCrawlerCount();
		}
	}

}
